#! /bin/sh
# Script to build all of the GNU tools.
#
# Arguments are, in order:
# path to the source directory
# path to where it will be build
# path to where it will be installed
# -boot designates that this is a bootstrapping operation, build
#	a select list of components and install in an XENV - to be
#	used to build the components for distribution.

strip_XENV_from_cmds () {

  strip_subdir=$1
  strip_list="$2"

  for i in $strip_list; do
    mv $GNUDIST/$strip_subdir/$i $GNUDIST/$strip_subdir/$i.tmp
    sed -e "s|$XENV||g" $GNUDIST/$strip_subdir/$i.tmp \
	> $GNUDIST/$strip_subdir/$i
    rm -f $GNUDIST/$strip_subdir/$i.tmp
  done
}


build_list () {

  CURR_DIR=`pwd`
  cd $BUILDDIR

  for subdir in $sublist; do
    [ "$subdir" = "$SKIPGCC" ] && continue
    echo "sub-directory $subdir ... \c"
    [ -d $BUILDDIR/$subdir ] && rm -rf $BUILDDIR/$subdir
    mkdir $BUILDDIR/$subdir
    cd $BUILDDIR/$subdir
  
    ${CONFIG_SHELL} ${SRCDIR}/${subdir}/OSDBUILD > $LOGDIR/${subdir}.conflog 2>&1
    touch ${LOGDIR}/${subdir}.bad
    if [ $? -ne 0 ]; then
      echo ""
      echo "ERROR: Configuration in $subdir failed!"
    else
      echo "configured ... \c"
      case $subdir in
        gcc*) mkargs='bootstrap BOOT_CFLAGS=-O6' ;;
        *)    mkargs= ;;
      esac
      gmake $mkargs > $LOGDIR/${subdir}.build 2>&1
      if [ $? -ne 0 ]; then
        echo ""
        echo "ERROR: Build in $subdir failed!"
      else
        echo "built ... \c"

        case $subdir in
          binutils) mkargs='CC_FOR_BUILD=gcc' ;;
          autogen) mkargs="CLexe=$BUILDDIR/$subdir/columns/columns" ;;
          *)        mkargs= ;;
        esac

        if [ -z "$XENV_INST" ]; then
          # real build for release
          [ -d ${GNUDIST}/${subdir} ] && rm -fr ${GNUDIST}/${subdir}
          mkdir ${GNUDIST}/${subdir}

          gmake install $mkargs DESTDIR=${GNUDIST}/${subdir} > $LOGDIR/${subdir}.inst 2>&1
          status=$?
          cmd_list=''
          case $subdir in
            autoconf) cmd_list="/usr/gnu/bin/autom4te /usr/gnu/bin/autoupdate"
          esac
          strip_XENV_from_cmds  $subdir "$cmd_list"

        else
          # XENV build
          gmake install $mkargs DESTDIR=/ > $LOGDIR/${subdir}.inst 2>&1
          status=$?
        fi
        if [ $status -ne 0 ]; then
          echo ""
          echo "ERROR: Install in $subdir failed!"
        else
          echo "installed"
          rm ${LOGDIR}/${subdir}.bad
        fi
      fi
    fi
  done

  cd $CURR_DIR
}


SCOABSPATH=1; export SCOABSPATH
CONFIG_SHELL=/usr/bin/bash; export CONFIG_SHELL
PATH=/usr/gnu/bin:$PATH; export PATH
OSVER=`uname -v`
case "$OSVER" in
  5.0.*)
    OSDIR=osr5
    SKIPGCC="gcc4"
    ;;
  6.0.*)
    OSDIR=osr6
    SKIPGCC="gcc2"
    ;;
  8* | 7*)
    OSDIR=uw7
    SKIPGCC="gcc2"
    ;;
esac

BOOT=0
if [ $# -eq 4 ]; then
  if [ "$4" != "-boot" ]; then
    echo "Usage: BUILD.sh /path/to/src /path/to/bld /path/to/install -boot"
  else
    BOOT=1
  fi
elif [ $# -ne 3 ]; then
  echo "Usage: BUILD.sh /path/to/src /path/to/bld /path/to/install -boot"
  exit 1
fi

SRCDIR=$1
BLDDIR=$2
GNUDIST=$3; export GNUDIST

if [ ! -d $SRCDIR -o ! -f $SRCDIR/BUILD.sh ]; then
  echo "Error: Source directory invalid "
  exit 1
fi

if [ ! -d $BLDDIR ]; then
  echo "Error: Build directory does not exist"
  exit 1
fi

if [ ! -d $GNUDIST ]; then
  echo "Error: Install dirctory does not exist"
  exit 1
fi

[ -d $BLDDIR/$OSDIR ] || {
  mkdir $BLDDIR/$OSDIR
}

XENV=$BLDDIR/$OSDIR/xenv; export XENV
[ -d $XENV/usr/gnu/bin ] || mkdir -p $XENV/usr/gnu/bin
[ -d $XENV/usr/bin ] || mkdir -p $XENV/usr/bin


[ -d $BLDDIR/$OSDIR/LOGS ] || {
  mkdir $BLDDIR/$OSDIR/LOGS
}

BUILDDIR=${BLDDIR}/${OSDIR}; export BUILDDIR
LOGDIR=${BUILDDIR}/LOGS; export LOGDIR
OSDBUILDTOP=${SRCDIR}/OSDBUILD.top.${OSDIR}; export OSDBUILDTOP

PATH=$XENV/usr/gnu/bin:$PATH
export PATH

cd $SRCDIR

unset XENV_INST
if [ $BOOT -eq 1 ]; then
  echo Populating the XENV ....................
  
  rm -rf $XENV
  XENV_INST=$XENV; export XENV_INST
  # list of packages needed to build other packages.  This may change
  # from release to release.

  sublist="make m4 autoconf automake autogen libtool bison texinfo gawk flex"

  echo " "
  echo ".... Saving copy of XENV source  "
  find $sublist | cpio -oc > XENV.src.cpio
  echo " "

  build_list;

  echo " "
  echo Finished populating XENV .........................
  echo " "
  echo ".... Restoring XENV original source"
  rm -rf $sublist
  cpio -icdum < XENV.src.cpio
  rm XENV.src.cpio
  echo " "

  unset XENV_INST
fi

sublist=`echo [a-z]*`
[ -n "$GNUBUILD" ] && sublist=$GNUBUILD

build_list

# Copy specific versions of the licenses into $DIST/licenses for 
# install in /usr/gnu/share
#
# This list may need revised as addtional licenses apply or later versions
# are released.

  mkdir -p $GNUDIST/licenses/usr/gnu/share
  cp $SRCDIR/cvs/COPYING	$GNUDIST/licenses/usr/gnu/share/COPYING.v1
  cp $SRCDIR/coreutils/COPYING	$GNUDIST/licenses/usr/gnu/share/COPYING.v2
  cp $SRCDIR/autogen/COPYING	$GNUDIST/licenses/usr/gnu/share/COPYING.v3
  cp $SRCDIR/flex/COPYING	$GNUDIST/licenses/usr/gnu/share/COPYING.flex
  cp $SRCDIR/cvs/COPYING.LIB	$GNUDIST/licenses/usr/gnu/share/COPYING.LIB.v2
  cp $SRCDIR/bc/COPYING.LIB	$GNUDIST/licenses/usr/gnu/share/COPYING.LIB.v2.1

echo "All done. Review logs in $LOGDIR"

